;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                           ;;
;;(@* "Programming Modes")                                   ;;;;;;
;;                                                           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Generic Programming Modes

(defun my-generic-prog-modes-hook ()
  "Sets up environment useful for a variety of programming modes.
   Add this to the corresponding hooks."
  (highlight-attn-words)
  (turn-on-local-comment-auto-fill))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; IDE Configuration CEDET/ECB/JDEE  (necessary files loaded in imports.el)

(defun my-semantic-init-hook ()
  (global-semantic-idle-scheduler-mode '1)
  (semantic-load-enable-excessive-code-helpers)
  (require 'semantic-ia))

(add-hook 'semantic-init-hook 'my-semantic-init-hook)

(eval-after-load 'senator   ;; alternatively could put this in a hook
  '(progn
     (define-key senator-mode-map "c"    'semantic-ia-describe-class)
     (define-key senator-mode-map "d"    'semantic-ia-show-doc)
     (define-key senator-mode-map "e"    'eassist-list-methods)
     (define-key senator-mode-map "F"    'semantic-ia-fast-jump)
     (define-key senator-mode-map "s"    'semantic-symref)
     (define-key senator-mode-map "S"    'semantic-symref-symbol)
     (define-key senator-mode-map "x"    'semantic-symref-regexp)
     (define-key senator-mode-map "\C-k" 'senator-kill-tag)
     (define-key senator-mode-map "\M-c" 'senator-copy-tag)
     (define-key senator-mode-map "\C-w" nil)
     (define-key senator-mode-map "\M-w" nil)
     ))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Emacs-Lisp Mode

(defun my-emacs-lisp-mode-hook ()
  (local-set-key "\C-c\C-e" 'eval-defun)
  (local-set-key "\C-c\C-o" 'outline-minor-mode)
  (local-set-key "\C-c\C-s" 'eval-last-sexp)
  (defun emacs-lisp-outline-minor-setup ()
    (setq outline-regexp ";;; \\|;; \\|(....")
    (local-set-key "\C-c\C-m" outline-mode-prefix-map)
    (when (featurep 'org)
      (define-key outline-minor-mode-map [(control tab)] 'org-cycle)
      (define-key outline-minor-mode-map [(shift tab)]   'org-global-cycle))) 
  (make-local-variable 'outline-minor-mode-hook)
  (add-hook 'outline-minor-mode-hook 'emacs-lisp-outline-minor-setup))

(add-hook 'emacs-lisp-mode-hook 'highlight-attn-words)
(add-hook 'emacs-lisp-mode-hook 'my-emacs-lisp-mode-hook)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Common C Modes (cc-mode) (C, C++, Java)

(autoload 'c-mode    "cc-mode" "C Editing Mode" t)    ;; Use cc-mode for all C editing
(autoload 'c++-mode  "cc-mode" "C++ Editing Mode" t)
(autoload 'java-mode "cc-mode" "Java Editing Mode" t)

(defun my-c-mode-common-hook ()
  (if (not (assoc "CRG" c-style-alist))
      (c-add-style "CRG" (cdr c-crg-style)))
  (c-set-style "CRG")
  (if (assq 'c-auto-hungry-string minor-mode-alist)   ;; Inhibit minor mode notation
      (setq minor-mode-alist
	    (cons '(c-auto-hungry-string "") (remove-matching-elements 'c-auto-hungry-string minor-mode-alist 'eq-car))))
  (local-set-key "\C-c\C-a" 'c-beginning-of-statement)
  (local-set-key "\C-c\C-e" 'c-end-of-statement)
  (local-set-key "\C-c\C-b" 'c-backward-conditional)
  (local-set-key "\C-c\C-f" 'c-forward-conditional)
  (local-set-key "\C-c\C-c" 'comment-region)
  (local-set-key "\M-q"     'c-fill-paragraph)
  (local-set-key "\C-c\C-d" 'c-down-conditional)
  (local-set-key "\C-c\C-u" 'c-up-conditional)
  (local-set-key "\C-c\C-i" 'c-indent-exp)
  (local-set-key "\C-c\C-q" 'c-indent-defun)
  (local-set-key "\C-c\C-m" 'compile);; m for make
  (local-set-key "\C-c\C-n" 'up-list)
  (local-set-key "\C-c\C-p" 'c-up-list-neg)
  (local-set-key "\C-c\C-o" 'outline-minor-mode)
  (local-set-key "\C-c\C-t" 'c-set-offset)
  (local-set-key "\C-c\C-x" 'c-macro-expand)
  (local-set-key [f4]       'c-fill-paragraph)
  )
  
(add-hook 'c-mode-common-hook 'highlight-attn-words)
(add-hook 'c-mode-common-hook 'my-c-mode-common-hook)

(defvar c-crg-style
  '("CRG"
    (c-basic-offset . 4)
    (c-comment-only-line-offset . (0 . 0))
    (c-offsets-alist . ((statement-block-intro . +)
			(substatement-open . 0)
			(label . 0)
			(case-label . *)
			(access-label . /)
			(statement-case-intro . *)
			(statement-cont . +)
			(cpp-macro . 0)
                        (inline-open . 0)
			(inher-intro . c-lineup-inher-intro)
			))
    (c-auto-newline . t)
    (c-hanging-braces-alist . ((brace-list-open)
			       (brace-list-close)
			       (substatement-open before after)
			       (block-close before after)))
    (c-hanging-comment-ender-p . nil)
    (c-cleanup-list . '(scope-operator list-close-comma))
    )
  "My personal formatting style for editing C/C++/Java source.")

(defun c-up-list-neg (arg)
  "Like up-list but with a negated arg"
  (interactive "P")
  (if (not arg)
      (up-list -1)
    (up-list arg)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; C-mode (old)

(defun my-c-mode-hook ()
  (setq indent-tabs-mode nil)
  (local-set-key "\C-c\C-z" 'compilation-close-compilation)
  )

(add-hook 'c-mode-hook 'my-c-mode-hook)

(defvar boc-mode-hook                    
  '((lambda ()
      (setq c-indent-level '4)
      (setq c-continued-statement-offset '4)
      (setq c-continued-brace-offset '-4)
      (setq c-brace-offset '0)
      (setq c-brace-imaginary-offset '0)
      (setq indent-tabs-mode 'nil) 
      (setq c-label-offset '-2)
      (setq c-auto-newline t)
      ))
  "Style hook for \"Boring Old C-mode\" in case
it is needed to run an earlier version of emacs.")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; C++-mode

(defun my-c++-mode-hook ()
  (setq indent-tabs-mode nil)
  (local-set-key "\C-c\C-z" 'compilation-close-compilation)
  )

(add-hook 'c++-mode-hook 'my-c++-mode-hook)

(defun c-lineup-inher-intro (langelem)
  "Line up inheritence intro : two characters past class keyword"
  (+ c-basic-offset (/ c-basic-offset 2))) 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; java-mode

(defun my-java-mode-hook ()
  (setq indent-tabs-mode nil)
  )

(add-hook 'java-mode-hook 'my-java-mode-hook)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; compilation-mode

(defun my-compilation-mode-hook ()
  (local-set-key "\C-c\C-z" 'compilation-close-compilation)
  )

(add-hook 'compilation-mode-hook 'my-compilation-mode-hook)

(defun compilation-close-compilation (arg)
  "Iconifies compilation pop-up frame.  With prefix arg,
kills *compilation* buffer and thus its dedicated frame,
if the latter exists."
  (interactive "P")
  (let* (err-msg buf)
    (setq buf (condition-case err-msg
		  (setq buf (compilation-find-buffer))
		(error (and (message (car (cdr err-msg))) nil))))
    (if (and buf (not arg))
	(let ((win (get-buffer-window buf t)))
	  (if win
	      (iconify-frame (window-frame win))))
      (if (and buf arg)
	  (kill-buffer buf))
      )))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; perl-mode


(defun my-perl-mode-hook ()
  (setq perl-indent-level '4)
  (setq perl-continued-statement-offset '4)
  (setq perl-continued-brace-offset '-4)
  (setq perl-brace-offset '0)
  (setq perl-brace-imaginary-offset '0)
  (setq indent-tabs-mode 'nil) 
  (setq perl-label-offset '-2)
  )

(add-hook 'perl-mode-hook 'highlight-attn-words)
(add-hook 'perl-mode-hook 'my-perl-mode-hook)

