;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                           ;;
;;(@* "Settings and Initialization")                         ;;
;;                                                           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Start-up configuration

(setq inhibit-startup-message 't)
(setq initial-major-mode 'fundamental-mode)
(setq initial-frame-alist my-frame-alist)  

(setq custom-file (expand-file-name "~/.emacs.d/emacs-custom.el")) ;; load in init.el

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Enable Disabled Commands

(put 'eval-expression  'disabled nil)
(put 'narrow-to-region 'disabled nil)
(put 'downcase-region  'disabled nil)
(put 'upcase-region    'disabled nil)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Basic Operation

(defun my-set-completion-mode (sym)
  "Choose among various intelligent completion methods
   according to symbol. Current choices are 'icicles, 'ido,
   'iswitchb, 'none."
   (cond
    ((equal sym 'icicles)
     (if (not (featurep 'icicles))
         (require 'icicles))
     (icicle-mode 1))
    ((equal sym 'ido)
     (ido-mode '1)
     (setq ido-case-fold t))
    ((equal sym 'iswitchb) ;;; Use smart buffer switching
     (iswitchb-mode '1)                      
     (setq iswitchb-case t))
    ))
   
    
(defun my-set-operating-state ()
  "Set Emacs to preferred modes, maps, and more."
  (setq-default case-fold-search 'nil)
  (setq-default indent-tabs-mode nil)
    
  (setq-default fill-column '72)
  (setq paragraph-start paragraph-separate)            ;;; Use blank lines to separate paragraphs by default
  (setq adaptive-fill-regexp "[ \t]*\\([>*%#]+ +\\)?") ;;; Fill around comment beginnings and yanked-messages
  (setq sentence-end-double-space nil)                 ;;; Allow frenchspacing
  (setq page-delimiter "^\\(\f\\|\n\n+\\)")            ;;; ATTN: Consider making it 3 or more

  (setq print-length 1024)                 ;;; Give more information in help
  (setq print-level  8)

  (setq default-major-mode 'org-mode)      ;; back to fundamental-mode if org-mode causes strange behavior??
  
  (setq display-time-24hr-format t)
  (display-time)                           ;;; Time on Mode Line
    
  (transient-mark-mode '1)                 ;;; Highlight region
  (if (fboundp 'menu-bar-mode)
      (menu-bar-mode  -1))                 ;;; Disable menu bar
  (if (fboundp 'tool-bar-mode)
      (tool-bar-mode  -1))                 ;;; Disable tool bar
  (setq-default scroll-bar-mode 'right)    ;;; Put scroll bars on the right
  (toggle-scroll-bar  '1)                  ;;; Use scroll bars
  (auto-compression-mode '1)               ;;; Auto view compressed files
  (turn-on-global-show-paren-mode)         ;;; show matching parens
  (setq indicate-empty-lines t)            ;;; show empty lines at end of file

  (setq browse-url-browser-function 'browse-url-firefox) ;; use new tabs in firefox
  (setq browse-url-firefox-new-window-is-tab t)
  (setq browse-url-new-window-flag  t)

  (my-set-completion-mode 'icicles)
  (my-set-global-keybindings)
  )

