;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                           ;;
;;(@* "TeX-related Modes")                                   ;;
;;                                                           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; AUC TeX support

(setq TeX-default-mode 'plain-TeX-mode)
(setq TeX-format-list  '(("AMSTEX" ams-tex-mode "\\\\document\\b")
			 ("LATEX" latex-mode "\\\\\\(documentstyle\\|documentclass\\)\\b")
			 ("TEX" plain-tex-mode ".")
			 ))

(setq TeX-expand-list
      '(
        ("%p" TeX-printer-query)
        ("%q"
         (lambda nil
           (TeX-printer-query TeX-queue-command 2)))
        ("%v"
         (lambda nil
           (TeX-style-check TeX-view-style)))
        ("%V"
         (lambda nil
           (TeX-style-check TeX-output-view-style)))
        ("%r"
         (lambda nil
           (TeX-style-check TeX-print-style)))
        ("%l"
         (lambda nil
           (TeX-style-check LaTeX-command-style)))
        ("%s" file nil t)
        ("%t" file t t)
        ("%n" TeX-current-line)
        ("%d" file "dvi" t)
        ("%f" file "ps" t)
        ("%a" file "pdf" t)
        ("%b" TeX-current-file-name-nondirectory)
        ("%m"
         (lambda ()
           (let ((targets (my-get-makefile-targets))) ;; ATTN: need to write this function
             (completing-read "Target: " targets)) ))
        ))

(setq TeX-command-list
  (list (list "TeX" "tex '\\nonstopmode\\input %t'" 'TeX-run-TeX nil t)
	(list "TeX Interactive" "tex %t" 'TeX-run-interactive nil t)
	(list "LaTeX" "%l '\\nonstopmode\\input{%t}'"
	      'TeX-run-LaTeX nil t)
	(list "LaTeX Interactive" "%l %t" 'TeX-run-interactive nil t)
	(list "LaTeX2e" "latex2e '\\nonstopmode\\input{%t}'"
	      'TeX-run-LaTeX nil t)
	(if (or window-system (getenv "DISPLAY"))
	    (list "View" "xdvi %d " 'TeX-run-silent t nil)
	  (list "View" "dvi2tty -q -w 132 %s " 'TeX-run-command t nil))
	(if (or window-system (getenv "DISPLAY"))
	    (list "GhostView" "gv %f " 'TeX-run-silent t nil)
	  (list "GhostView" "dvi2tty -q -w 132 %s " 'TeX-run-command t nil))
	(if (or window-system (getenv "DISPLAY"))
	    (list "NoteView" "gv --swap %f " 'TeX-run-silent t nil)
	  (list "NoteView" "dvi2tty -q -w 132 %s " 'TeX-run-command t nil))
	(if (or window-system (getenv "DISPLAY"))
	    (list "AcroView" "acroread %a " 'TeX-run-silent t nil)
	  (list "AcroView" "dvi2tty -q -w 132 %s " 'TeX-run-command t nil))
	(list "File" "dvips %d -o %f " 'TeX-run-command t nil)
	(list "PDF" "dvipdf %d %a " 'TeX-run-command t nil)
	(list "Make" "make %m " 'TeX-run-command t nil)
	(list "Print" "%p " 'TeX-run-command t nil)
	(list "Queue" "%q" 'TeX-run-background nil nil)
	(list "BibTeX" "bibtex %s" 'TeX-run-BibTeX nil nil)
	(list "Index" "makeindex %s" 'TeX-run-command nil t)
	;; (list "Check" "chktex -v3 %s" 'TeX-run-compile nil t)
	;; Uncomment the above line and comment out the next line to
	;; use `chktex' instead of `lacheck'. 
	(list "Check" "lacheck %s" 'TeX-run-compile nil t)
	(list "Spell" "<ignored>" 'TeX-run-ispell-on-document nil nil)
	(list "Other" "" 'TeX-run-command t t)
	;; Not part of standard TeX.
	(list "Info" "makeinfo %t" 'TeX-run-compile nil t)
	(list "zAmSTeX" "amstex '\\nonstopmode\\input %t'"
	      'TeX-run-TeX nil t)) )


(defun my-plain-TeX-mode-hook ()
  (local-set-key "\C-c\C-e" 'LaTeX-environment)
  (local-set-key "$" 'self-insert-command)
  (setq LaTeX-default-environment "center")
  (setq TeX-font-list '( (?\C-b "{\\bf " "}")
			 (?\C-c "{\\sc " "}")
			 (?\C-e "{\\em " "\\/}")
			 (?\C-i "{\\it " "\\/}")
			 (?\C-m "{\\mb " "}")
			 (?\C-r "{\\rm " "}")
			 (?\C-s "{\\sl " "\\/}")
			 (?\C-t "{\\tt " "}")
			 (?\C-d "" "" t)
			 ))
  )
  
(add-hook 'plain-TeX-mode-hook 'highlight-attn-words)
(add-hook 'plain-TeX-mode-hook 'turn-on-flyspell) 
(add-hook 'plain-TeX-mode-hook 'turn-on-tex-parser-ispell)
(add-hook 'plain-TeX-mode-hook 'my-plain-TeX-mode-hook)

(add-hook 'LaTeX-mode-hook 'highlight-attn-words)
(add-hook 'LaTeX-mode-hook 'turn-on-tex-parser-ispell)
(add-hook 'LaTeX-mode-hook 'turn-on-flyspell) 
(autoload 'LaTeX-environment "latex" "Make Environment Using AucTeX commands" t)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; BibTeX-mode

(setq bibtex-mode-hook
      '((lambda ()
	  (local-set-key "\"" 'self-insert-command)
	  (setq bibtex-include-OPTannote nil)
	  (setq bibtex-include-OPTcrossref nil)
	  )))

